<?php

/*
## AirPay Component v1.0
##
## Developer
## AirPay - http://www.airpay.lv
## Vitaliy Andrianov - v@airpay.lv
##
## Date: 2012.02.29
*/

 defined('_JEXEC') or die('Restricted access');

 include(dirname(__FILE__)."/airpay.class.php");

 class pm_airpay extends PaymentRoot{
    
	function loadLanguageFile($langtag = ""){
		$lang = &JFactory::getLanguage();
		if ($langtag=="") $langtag = $lang->getTag();

		if(!file_exists(dirname(__FILE__) . '/lang/'.$langtag.'.php'))	$langtag = "en-GB";

		require_once (dirname(__FILE__) . '/lang/'.$langtag.'.php');
	}

	function showPaymentForm($params, $pmconfigs){
		include(dirname(__FILE__)."/paymentform.php");
	}

	//function call in admin
	function showAdminFormParams($params){
		$array_params = array('testmode', 'merchant_id', 'merchant_secret', 'transaction_end_status', 'transaction_pending_status', 'transaction_failed_status', 'transaction_refunded_status');

		foreach ($array_params as $key){
			if (!isset($params[$key])) $params[$key] = '';
		} 

		$orders = &JModel::getInstance('orders', 'JshoppingModel'); //admin model

		pm_airpay::loadLanguageFile();
		include(dirname(__FILE__)."/adminparamsform.php");	  
	}

	function checkTransaction($pmconfigs, $order, $act) {

	       	$jshopConfig = &JSFactory::getConfig();

		$airpay = new airpay( $pmconfigs['merchant_id'], $pmconfigs['merchant_secret'] );

		if ($act == "notify") {

			if (!$ret = $airpay->response($_POST, 'status')) {
				// error
				// update Your logs for the investigation
				foreach($_POST as $k => $v) $tmp_post = $k."=".$v."\n";
				saveToLog("payment.log", "AirPay:status: check failed - invalid POST: ".$tmp_post."\n");
			} else {
				# $ret array description
				#
				# Common data submitted by merchant:
				# [version] 			- version of the AirPay used
				# [merchant_id] 		- merchant ID
				# [mc_transaction_id] 		- merchant invoice value
				# [transaction_id]		- transaction ID in AirPay system
				# [amount]			- amount in minor units
				# [currency]			- currency code
				# [description] 		- description
				# [optional1]			- optional fields
				#
				# Payment data
				# [payment_system] 		- alias of the payment system that was used
				# [status_id]			- status of the transaction
				# [payment_system_status]	- original status of the transaction provided by the payment system above
				#
				# Sign
				# [hash] 			- hash sign
				#
				# Additional data
				# [is_reference_of] 		- presents if the transaction is a reference of any other transaction. used when refund happened
				#
				# Possible payment statuses
				# -1	- transaction expired with no try for the payment, means that the client just closed the AirPay window without proceeding the payment
				#  0	- transaction is created by payment() or payment_req() methods of the AirPay class, means that the AirPay window opened
				#  1	- transaction is successfull, means that the merchant can provide the service or product to the client
				#  2	- transaction is pending, means that the client have tried to pay and the payment system is still processing the payment
				#  3	- transaction is rejected, means that the client have no funds or cancelled the payment etc
				#  4	- transaction is expired, means that it didn't receive any status after the pending for a long time (36 hours). consider it as rejected.
				#  5	- transaction is refunded, means that the client received the funds back and Your account was charged for this amount

				// update Your local database according to the received data, send e-mail etc

				# Always check the following for successfull transactions:
				# the transaction was never successfull before (except it is refunded now)
				# the amount and currency is exactly what You expect to receive according to Your local database

				if ($order->order_status == $pmconfigs['transaction_end_status']) {
					return array(0, 'Error. Already processed (status: '.$ret['status_id'].'). Order ID '.$order->order_id);
				}

	        		if ($order->order_total * 100 != $ret['amount']){
					return array(0, 'Error amount. Order ID '.$order->order_id);
			        }

		        	if ($order->currency_code_iso != $ret['currency']){
        				return array(0, 'Error currency. Order ID '.$order->order_id);            
			        }

				if ($jshopConfig->savelog && $jshopConfig->savelogpaymentdata){
        	        		saveToLog("paymentdata.log", serialize($ret));
				}

				switch ($ret['status_id']) {
					case -1:
					case  3:
					case  4:
					case  5:
                			        saveToLog("payment.log", "Status rejected. Order ID ".$order->order_id." / ".$ret['transaction_id'].". Reason: ".$ret['status_id']);
		                        	return array(3, "AirPay: Rejected" );
						break;

					case  0:
					case  2:
                			        saveToLog("payment.log", "Status pending. Order ID ".$order->order_id." / ".$ret['transaction_id'].". Reason: ".$ret['status_id']);
		                        	return array(2, "AirPay: Pending" );
						break;

					case  1:
                			        saveToLog("payment.log", "Status OK. Order ID ".$order->order_id." / ".$ret['transaction_id'].". Reason: ".$ret['status_id']);
						return array(1, '');
						break;

					default:
                			        saveToLog("payment.log", "Status ERROR. Order ID ".$order->order_id." / ".$ret['transaction_id'].". Reason: ".$ret['status_id']);
						return array(0, "Error response. Order ID ".$order->order_id);  
				}

			}
		}

		return array(0, 'Error');
     
	}

	function showEndForm($pmconfigs, $order) {

	        $jshopConfig = &JSFactory::getConfig();	    
        	$item_name = sprintf(_JSHOP_PAYMENT_NUMBER, $order->order_number);

		$airpay = new airpay( $pmconfigs['merchant_id'], $pmconfigs['merchant_secret'] );

		if ($pmconfigs['testmode'])	$airpay->url = "https://www.airpayment.net/sandbox/gateway/";
		else				$airpay->url = "https://www.airpayment.net/new/gateway/";

	        $_country = &JTable::getInstance('country', 'jshop');
        	$_country->load($order->country);

		switch ($order->lang) {
			case 'ru-RU':	$lang = "RUS";	break;
			case 'lv-LV':	$lang = "LAT";	break;
			default:	$lang = "ENG";
		}

		$invoice = array(
			'amount'	=> $order->order_total * 100,						// minor units, e.g. 1 for 0.01
			'currency'	=> $order->currency_code_iso,						// currency code in ISO 4217
			'invoice'	=> $order->order_id,							// unique transaction value
			'language'	=> $lang,								// language: LAT, RUS, ENG
			'cl_fname'	=> $order->f_name == '' ? 'n/a' : $order->f_name,			// client's first name
			'cl_lname'	=> $order->l_name == '' ? 'n/a' : $order->l_name,			// client's last name
			'cl_email'	=> $order->email,							// client's e-mail address
			'cl_country'	=> $_country->country_code_2 == '' ? 'n/a' : $order->country_code_2,	// country code in ISO 3166-1-alpha-2
			'cl_city'	=> $order->city == '' ? 'n/a' : $order->city,				// city name
			'description'	=> $item_name,								// description of the transaction, visible to the client, e.g. description of the product
			'psys'		=> '', 									// payment system alias. empty for default or taken from $airpay->psystems
		);
       
?>

        <html>
        <body>
        <head>
            <meta http-equiv="content-type" content="text/html; charset=utf-8" />            
        </head>
	<body>

<?php
		$airpay->payment($invoice);
?>

        <?php print _JSHOP_REDIRECT_TO_PAYMENT_PAGE ?>

        <br>
        <script type="text/javascript">document.getElementById('pform').submit();</script>
        </body>
        </html>

<?php
        	die();
	}

	function getUrlParams($pmconfigs) {  

		$order_id = JRequest::getInt("order_id");

		// is it "return" from AirPay ?
		if (isset($_GET['transaction_id']) && isset($_GET['transaction_hash']) && isset($_GET['status'])) {
			
			$airpay = new airpay( $pmconfigs['merchant_id'], $pmconfigs['merchant_secret'] );

			if (!$ret = $airpay->response($_GET, 'return')) {
				$order_id = false;

				saveToLog("payment.log", "AirPay:return: invalid query");
				JError::raiseWarning(500, _JSHOP_ERROR_PAYMENT);    
  
			} else {
				## it is not recommended to update local database on this step
				## all the information here is just for the redirection of the client
				## status:
				## 1 = SUCCESS
				## 2 = CANCELLED

				$order_id = $ret['transaction_id'];

				if ($ret['status'] == 1) {
					// where SUCCESS does not means that the payment is successfull
					// You just have to tell the client that the payment is processing and there are still no errors

					## the redirection to Your local page is supposed to be here
					// header ("Location: http://www.merchant-website.com/thank_you.html");

					header( "Location: " .JURI::root(). "index.php?option=com_jshopping&controller=checkout&task=step7&act=return&js_paymentclass=pm_airpay&order_id=".$order_id );
					exit;
				}

				if ($ret['status'] == 2) {
					// status CANCELLED means that the client clicked "cancel" button somewhere

					## the redirection to Your local page is supposed to be here
					// header ("Location: http://www.merchant-website.com/cancelled.html");

					header( "Location: " .JURI::root(). "index.php?option=com_jshopping&controller=checkout&task=step7&act=cancel&js_paymentclass=pm_airpay&order_id=".$order_id );
					exit;
				}
			}

		}

		// is it "status" from AirPay ?
		if ($_GET['act'] == 'notify') {

			$airpay = new airpay( $pmconfigs['merchant_id'], $pmconfigs['merchant_secret'] );

			if (!$ret = $airpay->response($_POST, 'status')) {
				// error
				// update Your logs for the investigation

				foreach($_POST as $k => $v) $tmp_post = $k."=".$v."\n";
				saveToLog("payment.log", "AirPay:status: check failed - invalid POST: ".$tmp_post."\n");

				exit;
			} else {
				# $ret array description
				#
				# Common data submitted by merchant:
				# [mc_transaction_id] 		- merchant invoice value
				# [status_id]			- status of the transaction
				#
				# Additional data
				# [is_reference_of] 		- presents if the transaction is a reference of any other transaction. used when refund happened
				#
				# Possible payment statuses
				# -1	- transaction expired with no try for the payment, means that the client just closed the AirPay window without proceeding the payment
				#  0	- transaction is created by payment() or payment_req() methods of the AirPay class, means that the AirPay window opened
				#  1	- transaction is successfull, means that the merchant can provide the service or product to the client
				#  2	- transaction is pending, means that the client have tried to pay and the payment system is still processing the payment
				#  3	- transaction is rejected, means that the client have no funds or cancelled the payment etc
				#  4	- transaction is expired, means that it didn't receive any status after the pending for a long time (36 hours). consider it as rejected.
				#  5	- transaction is refunded, means that the client received the funds back and Your account was charged for this amount

				// just a check if the data is ok
				$order_id = $ret['mc_transaction_id'];
			}

		}

		// this will be executed when it's not return or notify from AirPay

		$params = array(); 
		$params['order_id'] 		= $order_id;		// this is what we set before
		$params['hash'] 		= "";			// no hash
		$params['checkHash'] 		= 0;			// no hash check
		$params['checkReturnParams'] 	= false;		// don't check "return" params

		return $params;
	}
    
 }
?>